@@ -84,17 +84,94 @@ function(file, output, text, renderer='HTML', renderer.options=NULL,
    invisible(ret)
 }
 
-.mimeType <- function(file){
-   if      (grepl('.png$',file,perl=TRUE,ignore.case=TRUE))
-      'image/png'
-   else if (grepl('.gif$',file,perl=TRUE,ignore.case=TRUE))
-      'image/gif'
-   else if (grepl('(.jpg$|.jpeg$)',file,perl=TRUE,ignore.case=TRUE))
-      'image/jpeg'
-   else if (grepl('.tiff?$',file,perl=TRUE,ignore.case=TRUE))
-      'image/tiff'
-   else
-      ''
+.MIMEMAP <- new.env()
+
+## most common web types
+.MIMEMAP$htm <- "text/html"
+.MIMEMAP$html <- "text/html"
+.MIMEMAP$css <- "text/css"
+.MIMEMAP$gif <- "image/gif"
+.MIMEMAP$jpg <- "image/jpeg"
+.MIMEMAP$jpeg <- "image/jpeg"
+.MIMEMAP$jpe <- "image/jpeg"
+.MIMEMAP$png <- "image/png"
+.MIMEMAP$js <- "application/x-javascript"
+.MIMEMAP$pdf <- "application/pdf"
+.MIMEMAP$svg <- "image/svg+xml"
+.MIMEMAP$swf <- "application/x-shockwave-flash"
+
+## markdown types
+.MIMEMAP$md <- "text/x-markdown"
+.MIMEMAP$mdtxt <- "text/x-markdown"
+.MIMEMAP$markdown <- "text/x-markdown"
+
+## other types we are likely to serve
+.MIMEMAP$xml <- "text/xml"
+.MIMEMAP$csv <- "text/csv"
+.MIMEMAP$ico <- "image/x-icon"
+.MIMEMAP$zip <- "application/zip"
+.MIMEMAP$bz <- "application/x-bzip"
+.MIMEMAP$bz2 <- "application/x-bzip2"
+.MIMEMAP$gz <- "application/x-gzip"
+.MIMEMAP$tar <- "application/x-tar"
+
+## yet more types...
+
+.MIMEMAP$shtml <- "text/html"
+.MIMEMAP$tsv <- "text/tab-separated-values"
+.MIMEMAP$tab <- "text/tab-separated-values"
+.MIMEMAP$dcf <- "text/debian-control-file"
+.MIMEMAP$txt <- "text/plain"
+.MIMEMAP$mml <- "text/mathml"
+
+.MIMEMAP$tif <- "image/tiff"
+.MIMEMAP$tiff <- "image/tiff"
+.MIMEMAP$bmp <- "image/bmp" 
+.MIMEMAP$ps <- "application/postscript"
+.MIMEMAP$eps <- "application/postscript"
+.MIMEMAP$dvi <-   "application/x-dvi"
+
+.MIMEMAP$atom <- "application/atom+xml"
+.MIMEMAP$rss <- "application/rss+xml"
+
+.MIMEMAP$doc <- "application/msword"
+.MIMEMAP$docx <- "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
+.MIMEMAP$odt <- "application/vnd.oasis.opendocument.text"
+.MIMEMAP$rtf <- "application/rtf"
+.MIMEMAP$xls <- "application/vnd.ms-excel"
+.MIMEMAP$xlsx <- "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
+.MIMEMAP$ods <- "application/x-vnd.oasis.opendocument.spreadsheet"
+.MIMEMAP$ppt <- "application/vnd.ms-powerpoint"
+.MIMEMAP$pps <- "application/vnd.ms-powerpoint"
+.MIMEMAP$pptx <- "application/vnd.openxmlformats-officedocument.presentationml.presentation"
+
+.MIMEMAP$sit <- "application/x-stuffit"
+.MIMEMAP$sxw <- "application/vnd.sun.xml.writer"
+
+.MIMEMAP$iso <- "application/octet-stream"
+.MIMEMAP$dmg <- "application/octet-stream"
+.MIMEMAP$exe <- "application/octet-stream"
+.MIMEMAP$dll <- "application/octet-stream"
+.MIMEMAP$deb <- "application/octet-stream"
+.MIMEMAP$otf <- "application/octet-stream"
+.MIMEMAP$xpi <- "application/x-xpinstall"
+
+.MIMEMAP$mp2 <- "audio/mpeg"
+.MIMEMAP$mp3 <- "audio/mpeg"
+
+.MIMEMAP$mpg <- "video/mpeg"
+.MIMEMAP$mpeg <- "video/mpeg"
+.MIMEMAP$flv <- "video/x-flv"
+
+.mimeType <- function(f){
+   f <- f[1]
+   fileExt <- function (x) 
+   {
+      pos <- regexpr("\\.([[:alnum:]]+)$", x)
+      ifelse(pos > -1L, tolower(substring(x, pos + 1L)), "")
+   }
+   ext <- fileExt(f)
+   ifelse(nchar(ext) > 1L && exists(ext,.MIMEMAP),.MIMEMAP[[ext]], "")
 }
 
 .b64EncodeFile <- function(inFile)
@@ -219,14 +296,14 @@ markdownToHTML <- function(file, output, text,
       # Need to scrub title more, e.g. strip html, etc.
       html <- sub("#!title#",title,html,perl=TRUE)
 
-      if ('highlight_code' %in% options || .requiresHighlighting(html)){
+      if ('highlight_code' %in% options && .requiresHighlighting(html)){
          highlight <- paste(readLines(system.file('resources/r_highlight.html',package='markdown')),collapse='\n')
       } else {
          highlight <- ''
       }
       html <- sub("#!r_highlight#",highlight,html,fixed=TRUE)
 
-      if ('mathjax' %in% options || .requiresMathJax(html)){
+      if ('mathjax' %in% options && .requiresMathJax(html)){
          mathjax <- paste(readLines(system.file('resources/mathjax.html',package='markdown')),collapse='\n')
       } else {
          mathjax <- ''
@@ -315,7 +392,7 @@ markdownHTMLOptions <- function(defaults=FALSE)
    if (!defaults)
       allOptions
    else
-      allOptions[seq(9,12)]
+      allOptions[seq(10,14)]
 }
 
 .onLoad <- function(libname,pkgname)
